# -*- coding: utf-8 -*-
"""
Created on Sun Mar 11 11:05:28 2018

@author: Gabriele Coiana
"""

import numpy as np
import sys,os
sys.path.insert(0, os.getcwd())


def get_parameters():
    """
    This function takes the input parameters from the file input.txt 
    and applies the right types to the variables
    """
    lista = []
    f = open('input', 'r', newline='\n')
    A = f.readlines()
    for string in A:
        index = string.find('=')+2
        lista.append(string[index:])
    
    latt = float(lista[1])
    
    
    m = lista[2]
    masses = np.fromstring(m, dtype=np.float, sep=',')
    mba,mti,mo = masses[0],masses[1],masses[2]
    
    N = lista[3]
    n = np.fromstring(N, dtype=np.int, sep=',')
    N1,N2,N3 = n[0],n[1],n[2]
    
    band = lista[4]
    a = np.fromstring(band, dtype=np.float, sep=',')
    num = len(a)/3
    ks = np.split(a,num)
    
    interp_wrong = lista[5]
    index = interp_wrong.find(' ')
    interp = interp_wrong[0:index]
    
    savefreq = int(lista[6])

     
    f.close()
    return latt, mba,mti,mo, N1,N2,N3, ks, interp, savefreq










#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Nov 13 16:00:25 2018

@author: Gabriele Coiana
"""
import numpy as np
from scipy.interpolate import interp1d

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import animation

def transf_frequencies(frequencies):
    freq = np.copy(frequencies)
    for i in range(len(freq)):
        for j in range(len(freq[0])):
            if (np.abs(np.imag(freq[i,j])) <= 4):
                freq[i,j] = np.real(frequencies[i,j])
            else:
                freq[i,j] = - np.abs(np.imag(frequencies[i,j]))       
    return freq 

def interp(kk,frequencies, index_0_withboundaries, method='linear'):
    acou = []
    optic = []
    index_acou = []
    freq = transf_frequencies(frequencies)
    
    for i in range(len(frequencies)):
        for j in range(len(frequencies[0])):
            if(np.abs(np.real(frequencies[i,j])) <= 0.01 and np.abs(np.imag(frequencies[i,j])) <= 0.01):
                index_acou.append(i)

    for i in index_acou:
        for j in range(len(index_0_withboundaries)-1):
            acou.append(interp1d(kk[index_0_withboundaries[j]:index_0_withboundaries[j+1]+1],frequencies[i,index_0_withboundaries[j]:index_0_withboundaries[j+1]+1],method))
            
    for i in range(len(frequencies)):
        if(i in index_acou):
            continue
        optic.append(interp1d(kk,freq[i,:],method))
        
    return index_acou, acou, optic

def plot(ks, kk, frequencies, interpol):
    fig,ax = plt.subplots()
    index_0 = []
    for i in range(len(ks)):
        if np.array_equal(ks[i],[0.,0.,0.]):
            index_0.append(i)
    
    index_0_withboundaries = index_0     
    if(index_0[-1] != len(kk)-1):
        index_0_withboundaries.append(len(kk)-1)
        
    if(index_0[0] != 0):
        index_0_withboundaries.insert(0,0)
    
    freq = transf_frequencies(frequencies)
    
    for i in range(len(frequencies)):
        ax.scatter(kk,freq[i,:],marker='o')
    
    
    if(interpol in ['cubic','quadratic','linear']):
        index_acou, acou, optic = interp(kk, frequencies, index_0_withboundaries, method=interpol)
        for i in range(len(index_acou)):
            for j in range(len(index_0_withboundaries)-1):
                xj = np.arange(kk[index_0_withboundaries[j]],kk[index_0_withboundaries[j+1]],0.01)
                ax.plot(xj,acou[i*(len(index_0_withboundaries)-1)+j](xj),c='r')
                
        x = np.arange(kk[0],kk[-1],0.01)
        for i in range(len(frequencies)-3):
            ax.plot(x,optic[i](x),c='r')
    
    
    Points = {'$\Gamma$':[0,0,0],'X':[0.5,0,0],'M':[0.5,0.5,0],'R':[0.5,0.5,0.5],'boh':[0,0.5,0], 'Z':[0,0,0.5]}
    x_labels = []
    for kpoint in ks:
        a = 0
        for element in Points.items():
            a = a + 1
            if (np.array_equal(kpoint , element[1])):
                x_labels.append(element[0])
                break
            if (a==len(Points.items())):
                x_labels.append(' ')
    
    ax.set_xlabel('Wave vector')
    ax.set_ylabel('Frequency [Thz]')           
    plt.xticks(kk,x_labels)
    ax.axhline(0, linestyle='--', color='b')
    plt.grid(axis='y')
    plt.show()
    fig.savefig('dispersion.pdf')
    return







#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 12 18:21:44 2018

@author: Gabriele Coiana
"""
import numpy as np

import sys,os
sys.path.insert(0, os.getcwd())

import params
import dynam
import plot
import bz
import cell

# =============================================================================
# Parameters
a = params.get_parameters()[0]
mba, mti, mo = params.get_parameters()[1:4]
N1,N2,N3 = params.get_parameters()[4:7]
kinput = params.get_parameters()[7::][0]
interp = params.get_parameters()[8]
savefreq = params.get_parameters()[9]

N1N2N3 = N1*N2*N3 # Number of cells
N = N1*N2*N3*5    # Number of atoms
# =============================================================================
print('\nPho-NON-py')
print()
print('\nCalculating the k points...')
ks, ks_scaled, kk = bz.getk(N1,N2,N3,kinput)
print('Ok babe, Im gonna take all of these kpoints (scaled):\n',ks_scaled)


print('\nComputing the stiffness matrix...')
K = dynam.K(N1N2N3)
print('\tK is equilibrated to within ',np.max(np.abs(np.sum(K,axis=1))))

print('\nCalculating dynamical matrixes...')
FREQ = []
EIGVEC = []
mode = str(sys.argv[1])
for kpoint in ks:
    D = dynam.D(kpoint,K,N1,N2,N3,mba,mti,mo,mode)
    
    ll = np.matrix(D)
    print('\t',str(kpoint),' Hermitian: ',np.allclose(ll,ll.getH(),rtol=0.01,atol=0.01))
    
    eigvals, eigvec = np.linalg.eig(D)
    idx = eigvals.argsort()[::-1]   
    eigvals = eigvals[idx]  
    eigvec = eigvec[:,idx]
    freq = (np.sqrt(eigvals)/(2*np.pi))
    FREQ.append(freq)
    EIGVEC.append(eigvec)
print()


FREQ = np.array(FREQ).T

if(savefreq):
    np.savetxt('frequencies',np.column_stack([FREQ.real,FREQ.imag]))
    np.savetxt('frequencies_niceview',FREQ, fmt='%-1.2d')
    for i in range(len(EIGVEC)):
        with open('eigvectors','ab') as f: 
            np.savetxt(f,np.column_stack([EIGVEC[i].real, EIGVEC[i].imag]))



plot.plot(ks_scaled,kk, FREQ,interp)



























#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Mar  2 11:33:31 2019

@author: gabriele
"""
from matplotlib import animation
import numpy as np
import sys

import plot
import cell
import params

def animate(frame_BaTiO3,plotBa,plotTi,plotO):
    plotBa.collections = []
    plotTi.collections = []
    plotO.collections = []
    frame_Ba, frame_Ti, frame_O = frame_BaTiO3[0:3],frame_BaTiO3[3:6],frame_BaTiO3[6::]
    plotBa._offsets3d = ([frame_Ba[0]],[frame_Ba[1]],[frame_Ba[2]])
    plotTi._offsets3d = ([frame_Ti[0]],[frame_Ti[1]],[frame_Ti[2]])
    plotO._offsets3d = (frame_O[0:3],frame_O[3:6],frame_O[6::])
    return

a = params.get_parameters()[0]
eigvec = np.loadtxt('eigvectors')
eigfreq = np.loadtxt('frequencies')
EIGVEC_real = []
EIGVEC_imag = []
for i in range(0,len(eigvec),15):
    EIGVEC_real.append(eigvec[i:i+15,0:15])
    EIGVEC_imag.append(eigvec[i:i+15,15::])


k = sys.argv[1]
j = sys.argv[2]

freq = eigfreq[j,k]
eigvec_toplot = EIGVEC_real[k][j]




t = np.arange(0,2*np.pi,0.1*np.pi)
time_function = 1*np.sin(t)
BaTiO3_0 = cell.BaTiO3(a)
BaTiO3_0.supercell(1,1,1)
Ba_0, Ti_0, O1_0, O2_0, O3_0 = BaTiO3_0.Ba, BaTiO3_0.Ti, BaTiO3_0.O1, BaTiO3_0.O2, BaTiO3_0.O3
O_0 = np.hstack((O1_0,O2_0,O3_0))

Ba = np.repeat(Ba_0,len(time_function),axis=0) + (eigvec_toplot[0:3].reshape(3,1)*time_function).T
Ti = np.repeat(Ti_0,len(time_function),axis=0) + (eigvec_toplot[3:6].reshape(3,1)*time_function).T
O = np.repeat(O_0,len(time_function),axis=0) + (eigvec_toplot[6::].reshape(9,1)*time_function).T
BaTiO3 = np.hstack((Ba,Ti,O))

fig = plt.figure()
ax = fig.add_subplot(111,projection='3d')
plotBa = ax.scatter(Ba[0,0],Ba[0,1],Ba[0,2],s=137*50)
plotTi = ax.scatter(Ti[0,0],Ti[0,1],Ti[0,2],s=47*50)
plotO = ax.scatter(O[0,0:3],O[0,3:6],O[0,6::],s=15*50)

ani = animation.FuncAnimation(fig,animate,frames=BaTiO3,fargs=(plotBa,plotTi,plotO))




